@@ -3,7 +3,9 @@
  */
 
 #include <config.h>
-#include <tgmath.h>
+#ifndef sun
+  #include <tgmath.h>
+#endif
 #include <panel-applet.h>
 #include <egg-screen-help.h>
 #include "charpick.h"
@@ -17,7 +19,7 @@
 /* This is the default list used when starting charpick the first time */
 /* aacute, agrave, eacute, iacute, oacute, frac12, copy*/
 /* static const gchar *def_list = "áàéíñóœ©"; */
-static gunichar def_code[] = {225, 224, 233, 237, 241, 243, 189, 169, 1579, -1};
+static gunichar def_code[] = {225, 224, 233, 237, 241, 243, 189, 169, 1579, 8364, -1};
 
 /* aacute, agrave, acirc, atilde, auml. aring, aelig, ordf */
 /* static const gchar *a_list = "áàâãäåæª"; */
@@ -101,6 +103,24 @@ static const gunichar *chartable[] = {
 	misc_code
 };
 
+gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
 
 /* sets the picked character as the selection when it gets a request */
 static void
@@ -290,7 +310,24 @@ menuitem_activated (GtkMenuItem *menuitem, charpick_data *curr_data)
 	
 	curr_data->charlist = string;
 	build_table (curr_data);
-	panel_applet_gconf_set_string (applet, "current_list", curr_data->charlist, NULL);
+	if (key_writable (applet, "current_list"))
+		panel_applet_gconf_set_string (applet, "current_list", curr_data->charlist, NULL);
+}
+
+static void
+add_palette (GtkMenuItem *menuitem, charpick_data *curr_data)
+{
+	PanelApplet *applet = PANEL_APPLET (curr_data->applet);
+	GList *list = curr_data->chartable;
+	gchar *new;
+	
+	new = run_edit_dialog (NULL, _("Add Palette"));
+	if (!new)
+		return;
+		
+	list = g_list_append (list, new);
+	save_chartable (curr_data);
+  	populate_menu (curr_data);
 }
 
 void
@@ -321,6 +358,18 @@ populate_menu (charpick_data *curr_data)
 		list = g_list_next (list);
 	}
 	
+	menuitem = gtk_separator_menu_item_new ();
+	gtk_widget_show (menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	
+	menuitem = gtk_menu_item_new_with_label (_("Add a palette..."));
+	gtk_widget_show (menuitem);
+	g_signal_connect (G_OBJECT (menuitem), "activate",
+				   G_CALLBACK (add_palette), curr_data);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+	if ( ! key_writable (PANEL_APPLET (curr_data->applet), "chartable"))
+		gtk_widget_set_sensitive (menuitem, FALSE);
+	
 }
 
 static void
@@ -387,7 +436,7 @@ build_table(charpick_data *p_curr_data)
   if (p_curr_data->box)
     gtk_widget_destroy(p_curr_data->box);
     
-  if (p_curr_data->panel_vertical == TRUE)
+  if (p_curr_data->panel_vertical)
     box = gtk_vbox_new (FALSE, 0);
   else 
     box = gtk_hbox_new (FALSE, 0);
@@ -413,6 +462,7 @@ build_table(charpick_data *p_curr_data)
 		break;
   }
   gtk_container_add (GTK_CONTAINER (button), arrow);
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
   g_signal_connect (G_OBJECT (button), "clicked",
                             G_CALLBACK (chooser_button_clicked), p_curr_data);
@@ -478,7 +528,7 @@ build_table(charpick_data *p_curr_data)
 	index = CLAMP (index, 0, size_ratio-1);	
   	gtk_box_pack_start (GTK_BOX (row_box[index]), toggle_button[i], TRUE, TRUE, 0);
   }
-  
+ 
   gtk_container_add (GTK_CONTAINER(p_curr_data->applet), box);
   gtk_widget_show_all (p_curr_data->box);
 
@@ -662,7 +712,8 @@ get_chartable (charpick_data *curr_data)
 			curr_data->chartable = g_list_append (curr_data->chartable, string);
 		
 		}
-		save_chartable (curr_data);
+		if ( ! key_writable (PANEL_APPLET (curr_data->applet), "chartable"))
+			save_chartable (curr_data);
 	}
 	
 
@@ -715,8 +766,7 @@ charpicker_applet_fill (PanelApplet *applet)
   
   get_chartable (curr_data);
   
-  string  = panel_applet_gconf_get_string (applet, 
-  							     "current_list", NULL);
+  string  = panel_applet_gconf_get_string (applet, "current_list", NULL);
   if (string) {
   	list = curr_data->chartable;
   	while (list) {
@@ -724,10 +774,16 @@ charpicker_applet_fill (PanelApplet *applet)
   			curr_data->charlist = list->data;
   		list = g_list_next (list);
   	}
-  	g_free (string);
-  } 
-  else
+	/* FIXME: yeah leak, but this code is full of leaks and evil
+	   point shuffling.  This should really be rewritten
+	   -George */
+	if (curr_data->charlist == NULL)
+		curr_data->charlist = string;
+	else
+		g_free (string);
+  } else {
   	curr_data->charlist = curr_data->chartable->data;  
+  }
  
   curr_data->panel_size = panel_applet_get_size (applet);
   
